@@ -1271,6 +1271,76 @@ describe("buildConfigChain", function () {
           loadOptionsAsync({ filename, cwd: path.dirname(filename) }),
         ).rejects.toThrow(error);
       });
+
+      it("loadPartialConfig should return a list of files that were extended", () => {
+        const filename = fixture("config-files", "babelrc-extended", "src.js");
+
+        expect(
+          babel.loadPartialConfig({ filename, cwd: path.dirname(filename) }),
+        ).toEqual({
+          babelignore: fixture("config-files", ".babelignore"),
+          babelrc: fixture("config-files", "babelrc-extended", ".babelrc"),
+          config: undefined,
+          fileHandling: "transpile",
+          options: {
+            ...getDefaults(),
+            filename: filename,
+            cwd: path.dirname(filename),
+            root: path.dirname(filename),
+            comments: true,
+          },
+          files: new Set([
+            fixture("config-files", ".babelignore"),
+            fixture("config-files", "babelrc-extended", ".babelrc-extended"),
+            fixture("config-files", "babelrc-extended", ".babelrc"),
+          ]),
+        });
+      });
+
+      it("loadPartialConfig should return null when ignored", () => {
+        const filename = fixture("config-files", "babelignore", "src.js");
+
+        expect(
+          babel.loadPartialConfig({ filename, cwd: path.dirname(filename) }),
+        ).toBeNull();
+      });
+
+      it("loadPartialConfig should return a list of files when ignored with showIgnoredFiles option", () => {
+        const filename = fixture("config-files", "babelignore", "src.js");
+
+        expect(
+          babel.loadPartialConfig({
+            filename,
+            cwd: path.dirname(filename),
+            showIgnoredFiles: true,
+          }),
+        ).toEqual({
+          babelignore: fixture("config-files", "babelignore", ".babelignore"),
+          babelrc: undefined,
+          config: undefined,
+          fileHandling: "ignored",
+          options: {
+            ...getDefaults(),
+            filename: filename,
+            cwd: path.dirname(filename),
+            root: path.dirname(filename),
+          },
+          files: new Set([
+            fixture("config-files", "babelignore", ".babelignore"),
+          ]),
+        });
+      });
+
+      it("loadPartialConfig can be called with no arguments", () => {
+        const cwd = process.cwd();
+
+        try {
+          process.chdir(fixture("config-files", "babelrc-extended"));
+          expect(() => babel.loadPartialConfig()).not.toThrow();
+        } finally {
+          process.chdir(cwd);
+        }
+      });
     });
 
     it("should throw when `test` presents but `filename` is not passed", () => {
