@@ -65,15 +65,23 @@ type ParsingContext =
 const TSErrors = Object.freeze({
   ClassMethodHasDeclare: "Class methods cannot have the 'declare' modifier",
   ClassMethodHasReadonly: "Class methods cannot have the 'readonly' modifier",
+  ConstructorHasTypeParameters:
+    "Type parameters cannot appear on a constructor declaration.",
   DeclareClassFieldHasInitializer:
-    "'declare' class fields cannot have an initializer",
+    "Initializers are not allowed in ambient contexts.",
+  DeclareFunctionHasImplementation:
+    "An implementation cannot be declared in ambient contexts.",
   DuplicateModifier: "Duplicate modifier: '%0'",
   EmptyHeritageClauseType: "'%0' list cannot be empty.",
+  EmptyTypeArguments: "Type argument list cannot be empty.",
+  EmptyTypeParameters: "Type parameter list cannot be empty.",
   IndexSignatureHasAbstract:
     "Index signatures cannot have the 'abstract' modifier",
   IndexSignatureHasAccessibility:
     "Index signatures cannot have an accessibility modifier ('%0')",
   IndexSignatureHasStatic: "Index signatures cannot have the 'static' modifier",
+  IndexSignatureHasDeclare:
+    "Index signatures cannot have the 'declare' modifier",
   InvalidTupleMemberLabel:
     "Tuple members must be labeled with a simple identifier.",
   MixedLabeledAndUnlabeledElements:
@@ -86,10 +94,10 @@ const TSErrors = Object.freeze({
     "Private elements cannot have the 'abstract' modifier.",
   PrivateElementHasAccessibility:
     "Private elements cannot have an accessibility modifier ('%0')",
-  TemplateTypeHasSubstitution:
-    "Template literal types cannot have any substitution",
   TypeAnnotationAfterAssign:
     "Type annotations must come before default assignments, e.g. instead of `age = 25: number` use `age: number = 25`",
+  UnexpectedParameterModifier:
+    "A parameter property is only allowed in a constructor implementation.",
   UnexpectedReadonly:
     "'readonly' type modifier is only permitted on array and tuple literal types.",
   UnexpectedTypeAnnotation: "Did not expect a type annotation here.",
@@ -103,6 +111,7 @@ const TSErrors = Object.freeze({
 });
 
 // Doesn't handle "void" or "null" because those are keywords, not identifiers.
+// It also doesn't handle "intrinsic", since usually it's not a keyword.
 function keywordTypeFromName(
   value: string,
 ): N.TsKeywordTypeType | typeof undefined {
@@ -397,6 +406,9 @@ export default (superClass: Class<Parser>): Class<Parser> =>
         /* bracket */ false,
         /* skipFirstToken */ true,
       );
+      if (node.params.length === 0) {
+        this.raise(node.start, TSErrors.EmptyTypeParameters);
+      }
       return this.finishNode(node, "TSTypeParameterDeclaration");
     }
 
@@ -608,6 +620,8 @@ export default (superClass: Class<Parser>): Class<Parser> =>
 
       this.expect(tt.bracketL);
       node.typeParameter = this.tsParseMappedTypeParameter();
+      node.nameType = this.eatContextual("as") ? this.tsParseType() : null;
+
       this.expect(tt.bracketR);
 
       if (this.match(tt.plusMin)) {
@@ -760,17 +774,15 @@ export default (superClass: Class<Parser>): Class<Parser> =>
 
     tsParseTemplateLiteralType(): N.TsType {
       const node: N.TsLiteralType = this.startNode();
-      const templateNode = this.parseTemplate(false);
-      if (templateNode.expressions.length > 0) {
-        this.raise(
-          templateNode.expressions[0].start,
-          TSErrors.TemplateTypeHasSubstitution,
-        );
-      }
-      node.literal = templateNode;
+      node.literal = this.parseTemplate(false);
       return this.finishNode(node, "TSLiteralType");
     }
 
+    parseTemplateSubstitution(): N.TsType {
+      if (this.state.inType) return this.tsParseType();
+      return super.parseTemplateSubstitution();
+    }
+
     tsParseThisTypeOrThisTypePredicate(): N.TsThisType | N.TsTypePredicate {
       const thisKeyword = this.tsParseThisTypeNode();
       if (this.isContextual("is") && !this.hasPrecedingLineBreak()) {
@@ -1026,7 +1038,7 @@ export default (superClass: Class<Parser>): Class<Parser> =>
         const t: N.TsTypeAnnotation = this.startNode();
         this.expect(returnToken);
 
-        const asserts = this.tsTryParse(
+        const asserts = !!this.tsTryParse(
           this.tsParseTypePredicateAsserts.bind(this),
         );
 
@@ -1229,7 +1241,21 @@ export default (superClass: Class<Parser>): Class<Parser> =>
       this.checkLVal(node.id, BIND_TS_TYPE, undefined, "typescript type alias");
 
       node.typeParameters = this.tsTryParseTypeParameters();
-      node.typeAnnotation = this.tsExpectThenParseType(tt.eq);
+      node.typeAnnotation = this.tsInType(() => {
+        this.expect(tt.eq);
+
+        if (
+          this.isContextual("intrinsic") &&
+          this.lookahead().type !== tt.dot
+        ) {
+          const node: N.TsKeywordType = this.startNode();
+          this.next();
+          return this.finishNode(node, "TSIntrinsicKeyword");
+        }
+
+        return this.tsParseType();
+      });
+
       this.semicolon();
       return this.finishNode(node, "TSTypeAliasDeclaration");
     }
@@ -1465,42 +1491,45 @@ export default (superClass: Class<Parser>): Class<Parser> =>
         kind = "let";
       }
 
-      switch (starttype) {
-        case tt._function:
-          return this.parseFunctionStatement(
-            nany,
-            /* async */ false,
-            /* declarationPosition */ true,
-          );
-        case tt._class:
-          // While this is also set by tsParseExpressionStatement, we need to set it
-          // before parsing the class declaration to now how to register it in the scope.
-          nany.declare = true;
-          return this.parseClass(
-            nany,
-            /* isStatement */ true,
-            /* optionalId */ false,
-          );
-        case tt._const:
-          if (this.match(tt._const) && this.isLookaheadContextual("enum")) {
-            // `const enum = 0;` not allowed because "enum" is a strict mode reserved word.
-            this.expect(tt._const);
-            this.expectContextual("enum");
-            return this.tsParseEnumDeclaration(nany, /* isConst */ true);
-          }
-        // falls through
-        case tt._var:
-          kind = kind || this.state.value;
-          return this.parseVarStatement(nany, kind);
-        case tt.name: {
-          const value = this.state.value;
-          if (value === "global") {
-            return this.tsParseAmbientExternalModuleDeclaration(nany);
-          } else {
-            return this.tsParseDeclaration(nany, value, /* next */ true);
+      return this.tsInDeclareContext(() => {
+        switch (starttype) {
+          case tt._function:
+            nany.declare = true;
+            return this.parseFunctionStatement(
+              nany,
+              /* async */ false,
+              /* declarationPosition */ true,
+            );
+          case tt._class:
+            // While this is also set by tsParseExpressionStatement, we need to set it
+            // before parsing the class declaration to now how to register it in the scope.
+            nany.declare = true;
+            return this.parseClass(
+              nany,
+              /* isStatement */ true,
+              /* optionalId */ false,
+            );
+          case tt._const:
+            if (this.match(tt._const) && this.isLookaheadContextual("enum")) {
+              // `const enum = 0;` not allowed because "enum" is a strict mode reserved word.
+              this.expect(tt._const);
+              this.expectContextual("enum");
+              return this.tsParseEnumDeclaration(nany, /* isConst */ true);
+            }
+          // falls through
+          case tt._var:
+            kind = kind || this.state.value;
+            return this.parseVarStatement(nany, kind);
+          case tt.name: {
+            const value = this.state.value;
+            if (value === "global") {
+              return this.tsParseAmbientExternalModuleDeclaration(nany);
+            } else {
+              return this.tsParseDeclaration(nany, value, /* next */ true);
+            }
           }
         }
-      }
+      });
     }
 
     // Note: this won't be called unless the keyword is allowed in `shouldParseExportDeclaration`.
@@ -1620,11 +1649,7 @@ export default (superClass: Class<Parser>): Class<Parser> =>
       }
 
       const oldMaybeInArrowParameters = this.state.maybeInArrowParameters;
-      const oldYieldPos = this.state.yieldPos;
-      const oldAwaitPos = this.state.awaitPos;
       this.state.maybeInArrowParameters = true;
-      this.state.yieldPos = -1;
-      this.state.awaitPos = -1;
 
       const res: ?N.ArrowFunctionExpression = this.tsTryParseAndCatch(() => {
         const node: N.ArrowFunctionExpression = this.startNodeAt(
@@ -1640,8 +1665,6 @@ export default (superClass: Class<Parser>): Class<Parser> =>
       });
 
       this.state.maybeInArrowParameters = oldMaybeInArrowParameters;
-      this.state.yieldPos = oldYieldPos;
-      this.state.awaitPos = oldAwaitPos;
 
       if (!res) {
         return undefined;
@@ -1666,6 +1689,9 @@ export default (superClass: Class<Parser>): Class<Parser> =>
           );
         }),
       );
+      if (node.params.length === 0) {
+        this.raise(node.start, TSErrors.EmptyTypeArguments);
+      }
       // This reads the next token after the `>` too, so do this in the enclosing context.
       // But be sure not to parse a regex in the jsx expression `<C<number> />`, so set exprAllowed = false
       this.state.exprAllowed = false;
@@ -1709,9 +1735,12 @@ export default (superClass: Class<Parser>): Class<Parser> =>
 
       let accessibility: ?N.Accessibility;
       let readonly = false;
-      if (allowModifiers) {
+      if (allowModifiers !== undefined) {
         accessibility = this.parseAccessModifier();
         readonly = !!this.tsParseModifier(["readonly"]);
+        if (allowModifiers === false && (accessibility || readonly)) {
+          this.raise(startPos, TSErrors.UnexpectedParameterModifier);
+        }
       }
 
       const left = this.parseMaybeDefault();
@@ -1757,6 +1786,16 @@ export default (superClass: Class<Parser>): Class<Parser> =>
         this.finishNode(node, bodilessType);
         return;
       }
+      if (bodilessType === "TSDeclareFunction" && this.state.isDeclareContext) {
+        this.raise(node.start, TSErrors.DeclareFunctionHasImplementation);
+        if (
+          // $FlowIgnore
+          node.declare
+        ) {
+          super.parseFunctionBodyAndFinish(node, bodilessType, isMethod);
+          return;
+        }
+      }
 
       super.parseFunctionBodyAndFinish(node, type, isMethod);
     }
@@ -1771,6 +1810,39 @@ export default (superClass: Class<Parser>): Class<Parser> =>
       }
     }
 
+    tsCheckForInvalidTypeCasts(items: $ReadOnlyArray<?N.Expression>) {
+      items.forEach(node => {
+        if (node?.type === "TSTypeCastExpression") {
+          this.raise(
+            node.typeAnnotation.start,
+            TSErrors.UnexpectedTypeAnnotation,
+          );
+        }
+      });
+    }
+
+    toReferencedList(
+      exprList: $ReadOnlyArray<?N.Expression>,
+      isInParens?: boolean, // eslint-disable-line no-unused-vars
+    ): $ReadOnlyArray<?N.Expression> {
+      // Handles invalid scenarios like: `f(a:b)`, `(a:b);`, and `(a:b,c:d)`.
+      //
+      // Note that `f<T>(a:b)` goes through a different path and is handled
+      // in `parseSubscript` directly.
+      this.tsCheckForInvalidTypeCasts(exprList);
+      return exprList;
+    }
+
+    parseArrayLike(...args): N.ArrayExpression | N.TupleExpression {
+      const node = super.parseArrayLike(...args);
+
+      if (node.type === "ArrayExpression") {
+        this.tsCheckForInvalidTypeCasts(node.elements);
+      }
+
+      return node;
+    }
+
     parseSubscript(
       base: N.Expression,
       startPos: number,
@@ -1820,6 +1892,10 @@ export default (superClass: Class<Parser>): Class<Parser> =>
                 tt.parenR,
                 /* possibleAsync */ false,
               );
+
+              // Handles invalid case: `f<T>(a:b)`
+              this.tsCheckForInvalidTypeCasts(node.arguments);
+
               node.typeParameters = typeArguments;
               return this.finishCallExpression(node, state.optionalChainMember);
             } else if (this.match(tt.backQuote)) {
@@ -1931,11 +2007,13 @@ export default (superClass: Class<Parser>): Class<Parser> =>
         ) {
           node.importKind = "type";
           this.next();
-        } else {
-          node.importKind = "value";
         }
       }
 
+      if (!node.importKind) {
+        node.importKind = "value";
+      }
+
       const importNode = super.parseImport(node);
       /*:: invariant(importNode.type !== "TSImportEqualsDeclaration") */
 
@@ -2036,23 +2114,28 @@ export default (superClass: Class<Parser>): Class<Parser> =>
     parseClassMember(
       classBody: N.ClassBody,
       member: any,
-      state: { hadConstructor: boolean },
-      constructorAllowsSuper: boolean,
+      state: N.ParseClassMemberState,
     ): void {
       this.tsParseModifiers(member, ["declare"]);
       const accessibility = this.parseAccessModifier();
       if (accessibility) member.accessibility = accessibility;
       this.tsParseModifiers(member, ["declare"]);
 
-      super.parseClassMember(classBody, member, state, constructorAllowsSuper);
+      const callParseClassMember = () => {
+        super.parseClassMember(classBody, member, state);
+      };
+      if (member.declare) {
+        this.tsInDeclareContext(callParseClassMember);
+      } else {
+        callParseClassMember();
+      }
     }
 
     parseClassMemberWithIsStatic(
       classBody: N.ClassBody,
       member: N.ClassMember | N.TsIndexSignature,
-      state: { hadConstructor: boolean },
+      state: N.ParseClassMemberState,
       isStatic: boolean,
-      constructorAllowsSuper: boolean,
     ): void {
       this.tsParseModifiers(member, ["abstract", "readonly", "declare"]);
 
@@ -2073,19 +2156,16 @@ export default (superClass: Class<Parser>): Class<Parser> =>
             (member: any).accessibility,
           );
         }
+        if ((member: any).declare) {
+          this.raise(member.start, TSErrors.IndexSignatureHasDeclare);
+        }
 
         return;
       }
 
       /*:: invariant(member.type !== "TSIndexSignature") */
 
-      super.parseClassMemberWithIsStatic(
-        classBody,
-        member,
-        state,
-        isStatic,
-        constructorAllowsSuper,
-      );
+      super.parseClassMemberWithIsStatic(classBody, member, state, isStatic);
     }
 
     parsePostMemberNameModifiers(
@@ -2254,7 +2334,7 @@ export default (superClass: Class<Parser>): Class<Parser> =>
     parseClassProperty(node: N.ClassProperty): N.ClassProperty {
       this.parseClassPropertyAnnotation(node);
 
-      if (node.declare && this.match(tt.equal)) {
+      if (this.state.isDeclareContext && this.match(tt.eq)) {
         this.raise(this.state.start, TSErrors.DeclareClassFieldHasInitializer);
       }
 
@@ -2291,6 +2371,9 @@ export default (superClass: Class<Parser>): Class<Parser> =>
       allowsDirectSuper: boolean,
     ): void {
       const typeParameters = this.tsTryParseTypeParameters();
+      if (typeParameters && isConstructor) {
+        this.raise(typeParameters.start, TSErrors.ConstructorHasTypeParameters);
+      }
       if (typeParameters) method.typeParameters = typeParameters;
       super.pushClassMethod(
         classBody,
@@ -2496,7 +2579,11 @@ export default (superClass: Class<Parser>): Class<Parser> =>
     // Allow type annotations inside of a parameter list.
     parseAssignableListItemTypes(param: N.Pattern) {
       if (this.eat(tt.question)) {
-        if (param.type !== "Identifier") {
+        if (
+          param.type !== "Identifier" &&
+          !this.state.isDeclareContext &&
+          !this.state.inType
+        ) {
           this.raise(param.start, TSErrors.PatternIsOptional);
         }
 
@@ -2675,20 +2762,6 @@ export default (superClass: Class<Parser>): Class<Parser> =>
       return node.expression;
     }
 
-    toReferencedList(
-      exprList: $ReadOnlyArray<?N.Expression>,
-      isInParens?: boolean, // eslint-disable-line no-unused-vars
-    ): $ReadOnlyArray<?N.Expression> {
-      for (let i = 0; i < exprList.length; i++) {
-        const expr = exprList[i];
-        if (expr?.type === "TSTypeCastExpression") {
-          this.raise(expr.start, TSErrors.UnexpectedTypeAnnotation);
-        }
-      }
-
-      return exprList;
-    }
-
     shouldParseArrow() {
       return this.match(tt.colon) || super.shouldParseArrow();
     }
@@ -2738,4 +2811,14 @@ export default (superClass: Class<Parser>): Class<Parser> =>
 
       return param;
     }
+
+    tsInDeclareContext<T>(cb: () => T): T {
+      const oldIsDeclareContext = this.state.isDeclareContext;
+      this.state.isDeclareContext = true;
+      try {
+        return cb();
+      } finally {
+        this.state.isDeclareContext = oldIsDeclareContext;
+      }
+    }
   };
