@@ -360,6 +360,12 @@ export type Directive = NodeBase & {
 
 export type DirectiveLiteral = StringLiteral & { type: "DirectiveLiteral" };
 
+export type ImportAttribute = NodeBase & {
+  type: "ImportAttribute",
+  key: Identifier | StringLiteral,
+  value: StringLiteral,
+};
+
 // Expressions
 
 export type Super = NodeBase & { type: "Super" };
@@ -705,7 +711,7 @@ export type ClassBase = HasDecorators & {
 
 export type ClassBody = NodeBase & {
   type: "ClassBody",
-  body: Array<ClassMember | TsIndexSignature>, // TODO: $ReadOnlyArray
+  body: Array<ClassMember | StaticBlock | TsIndexSignature>, // TODO: $ReadOnlyArray
 };
 // | Placeholder<"ClassBody">;
 
@@ -719,6 +725,11 @@ export type ClassMemberBase = NodeBase &
     optional?: ?true,
   };
 
+export type StaticBlock = NodeBase & {
+  type: "StaticBlock",
+  body: Array<Statement>,
+};
+
 export type ClassMember =
   | ClassMethod
   | ClassPrivateMethod
@@ -842,7 +853,7 @@ export type ImportDeclaration = NodeBase & {
 
 export type ImportSpecifier = ModuleSpecifier & {
   type: "ImportSpecifier",
-  imported: Identifier,
+  imported: Identifier | StringLiteral,
 };
 
 export type ImportDefaultSpecifier = ModuleSpecifier & {
@@ -862,11 +873,13 @@ export type ExportNamedDeclaration = NodeBase & {
   source: ?Literal,
 
   exportKind?: "type" | "value", // TODO: Not in spec
+
+  assertions?: $ReadOnlyArray<ImportAttribute>,
 };
 
 export type ExportSpecifier = NodeBase & {
   type: "ExportSpecifier",
-  exported: Identifier,
+  exported: Identifier | StringLiteral,
   local: Identifier,
 };
 
@@ -1215,7 +1228,8 @@ export type TsKeywordTypeType =
   | "TSVoidKeyword"
   | "TSUndefinedKeyword"
   | "TSNullKeyword"
-  | "TSNeverKeyword";
+  | "TSNeverKeyword"
+  | "TSIntrinsicKeyword";
 export type TsKeywordType = TsTypeBase & {
   type: TsKeywordTypeType,
 };
@@ -1339,6 +1353,7 @@ export type TsMappedType = TsTypeBase & {
   typeParameter: TsTypeParameter,
   optional?: true | "+" | "-",
   typeAnnotation: ?TsType,
+  nameType: ?TsType,
 };
 
 export type TsLiteralType = TsTypeBase & {
@@ -1487,3 +1502,9 @@ export type ParseSubscriptState = {
   maybeAsyncArrow: boolean,
   stop: boolean,
 };
+
+export type ParseClassMemberState = {|
+  hadConstructor: boolean,
+  hadStaticBlock: boolean,
+  constructorAllowsSuper: boolean,
+|};
