@@ -1,10 +1,10 @@
-var Child =
-/*#__PURE__*/
-function (_Base) {
+var Child = /*#__PURE__*/function (_Base) {
   "use strict";
 
   babelHelpers.inherits(Child, _Base);
 
+  var _super = babelHelpers.createSuper(Child);
+
   function Child() {
     var _this;
 
