@@ -1401,6 +1401,13 @@ defineType("ExportAllDeclaration", {
     source: {
       validate: assertNodeType("StringLiteral"),
     },
+    assertions: {
+      optional: true,
+      validate: chain(
+        assertValueType("array"),
+        assertNodeType("ImportAttribute"),
+      ),
+    },
   },
 });
 
@@ -1464,6 +1471,13 @@ defineType("ExportNamedDeclaration", {
         },
       ),
     },
+    assertions: {
+      optional: true,
+      validate: chain(
+        assertValueType("array"),
+        assertNodeType("ImportAttribute"),
+      ),
+    },
     specifiers: {
       default: [],
       validate: chain(
@@ -1503,7 +1517,7 @@ defineType("ExportSpecifier", {
       validate: assertNodeType("Identifier"),
     },
     exported: {
-      validate: assertNodeType("Identifier"),
+      validate: assertNodeType("Identifier", "StringLiteral"),
     },
   },
 });
@@ -1559,6 +1573,13 @@ defineType("ImportDeclaration", {
   visitor: ["specifiers", "source"],
   aliases: ["Statement", "Declaration", "ModuleDeclaration"],
   fields: {
+    assertions: {
+      optional: true,
+      validate: chain(
+        assertValueType("array"),
+        assertNodeType("ImportAttribute"),
+      ),
+    },
     specifiers: {
       validate: chain(
         assertValueType("array"),
@@ -1611,7 +1632,7 @@ defineType("ImportSpecifier", {
       validate: assertNodeType("Identifier"),
     },
     imported: {
-      validate: assertNodeType("Identifier"),
+      validate: assertNodeType("Identifier", "StringLiteral"),
     },
     importKind: {
       // Handle Flowtype's extension "import {typeof foo} from"
@@ -1841,7 +1862,13 @@ defineType("TemplateLiteral", {
     expressions: {
       validate: chain(
         assertValueType("array"),
-        assertEach(assertNodeType("Expression")),
+        assertEach(
+          assertNodeType(
+            "Expression",
+            // For TypeScript template literal types
+            "TSType",
+          ),
+        ),
         function (node, key, val) {
           if (node.quasis.length !== val.length + 1) {
             throw new TypeError(
